//////// Standard Header file for the PIC16F628 device ////////////////
#device PIC16F628
#define __16F628
#nolist
//////// Program memory: 2048x14  Data RAM: 223  Stack: 8
//////// I/O: 16   Analog Pins: 0
//////// Data EEPROM: 128
//////// C Scratch area: 77   ID Location: 2000
//////// Fuses: LP,XT,HS,EC,NOWDT,WDT,NOPUT,PUT,PROTECT,PROTECT_50%
//////// Fuses: PROTECT_75%,NOPROTECT,NOBROWNOUT,BROWNOUT,MCLR,NOMCLR,LVP
//////// Fuses: NOLVP,ER_IO,INTRC_IO,ER,INTRC,CPD,NOCPD
////////
////////////////////////////////////////////////////////////////// I/O
// Discrete I/O Functions: SET_TRIS_x(), OUTPUT_x(), INPUT_x(),
//                         PORT_B_PULLUPS(), INPUT(),
//                         OUTPUT_LOW(), OUTPUT_HIGH(),
//                         OUTPUT_FLOAT(), OUTPUT_BIT()
// Constants used to identify pins in the above are:

#define PIN_A0  40
#define PIN_A1  41
#define PIN_A2  42
#define PIN_A3  43
#define PIN_A4  44
#define PIN_A5  45
#define PIN_A6  46
#define PIN_A7  47

#define PIN_B0  48
#define PIN_B1  49
#define PIN_B2  50
#define PIN_B3  51
#define PIN_B4  52
#define PIN_B5  53
#define PIN_B6  54
#define PIN_B7  55

////////////////////////////////////////////////////////////////// Useful defines
#define FALSE 0
#define TRUE 1

#define BYTE int
#define BOOLEAN short int

#define getc getch
#define fgetc getch
#define getchar getch
#define putc putchar
#define fputc putchar
#define fgets gets
#define fputs puts

////////////////////////////////////////////////////////////////// Control
// Control Functions:  RESET_CPU(), SLEEP(), RESTART_CAUSE()
// Constants returned from RESTART_CAUSE() are:
#define WDT_FROM_SLEEP  0
#define WDT_TIMEOUT     8
#define MCLR_FROM_SLEEP 16
#define NORMAL_POWER_UP 24


////////////////////////////////////////////////////////////////// Timer 0
// Timer 0 (AKA RTCC)Functions: SETUP_COUNTERS() or SETUP_TIMER0(),
//                              SET_TIMER0() or SET_RTCC(),
//                              GET_TIMER0() or GET_RTCC()
// Constants used for SETUP_TIMER0() are:
#define RTCC_INTERNAL   0
#define RTCC_EXT_L_TO_H 32
#define RTCC_EXT_H_TO_L 48

#define RTCC_DIV_1      8
#define RTCC_DIV_2      0
#define RTCC_DIV_4      1
#define RTCC_DIV_8      2
#define RTCC_DIV_16     3
#define RTCC_DIV_32     4
#define RTCC_DIV_64     5
#define RTCC_DIV_128    6
#define RTCC_DIV_256    7


#define RTCC_8_BIT      0

// Constants used for SETUP_COUNTERS() are the above
// constants for the 1st param and the following for
// the 2nd param:

////////////////////////////////////////////////////////////////// WDT
// Watch Dog Timer Functions: SETUP_WDT() or SETUP_COUNTERS() (see above)
//                            RESTART_WDT()
//
#define WDT_18MS        8
#define WDT_36MS        9
#define WDT_72MS       10
#define WDT_144MS      11
#define WDT_288MS      12
#define WDT_576MS      13
#define WDT_1152MS     14
#define WDT_2304MS     15

////////////////////////////////////////////////////////////////// Timer 1
// Timer 1 Functions: SETUP_TIMER_1, GET_TIMER1, SET_TIMER1
// Constants used for SETUP_TIMER_1() are:
//      (or (via |) together constants from each group)
#define T1_DISABLED         0
#define T1_INTERNAL         0x85
#define T1_EXTERNAL         0x87
#define T1_EXTERNAL_SYNC    0x83

#define T1_CLK_OUT          8

#define T1_DIV_BY_1         0
#define T1_DIV_BY_2         0x10
#define T1_DIV_BY_4         0x20
#define T1_DIV_BY_8         0x30

////////////////////////////////////////////////////////////////// Timer 2
// Timer 2 Functions: SETUP_TIMER_2, GET_TIMER2, SET_TIMER2
// Constants used for SETUP_TIMER_2() are:
#define T2_DISABLED         0
#define T2_DIV_BY_1         4
#define T2_DIV_BY_4         5
#define T2_DIV_BY_16        6

////////////////////////////////////////////////////////////////// CCP
// CCP Functions: SETUP_CCPx, SET_PWMx_DUTY
// CCP Variables: CCP_x, CCP_x_LOW, CCP_x_HIGH
// Constants used for SETUP_CCPx() are:
#define CCP_OFF                         0
#define CCP_CAPTURE_FE                  4
#define CCP_CAPTURE_RE                  5
#define CCP_CAPTURE_DIV_4               6
#define CCP_CAPTURE_DIV_16              7
#define CCP_COMPARE_SET_ON_MATCH        8
#define CCP_COMPARE_CLR_ON_MATCH        9
#define CCP_COMPARE_INT                 0xA
#define CCP_COMPARE_RESET_TIMER         0xB
#define CCP_PWM                         0xC
#define CCP_PWM_PLUS_1                  0x1c
#define CCP_PWM_PLUS_2                  0x2c
#define CCP_PWM_PLUS_3                  0x3c
long CCP_1;
#byte   CCP_1    =                      0x15
#byte   CCP_1_LOW=                      0x15
#byte   CCP_1_HIGH=                     0x16
////////////////////////////////////////////////////////////////// UART
// Constants used in setup_uart() are:
// FALSE - Turn UART off
// TRUE  - Turn UART on
////////////////////////////////////////////////////////////////// COMP
// Comparator Variables: C1OUT, C2OUT
// Constants used in setup_comparators() are:
#define A0_A3_A1_A2  0xfff04
#define A0_A2_A1_A2  0x7ff03
#define NC_NC_A1_A2  0x6ff05
#define NC_NC_NC_NC  0x0ff07
#define A0_VR_A1_VR  0x3ff02
#define A3_VR_A2_VR  0xcff0A
#define A0_A2_A1_A2_OUT_ON_A3_A4 0x7e706
#define A3_A2_A1_A2  0xeff09

#bit C1OUT = 0x1f.6
#bit C2OUT = 0x1f.7

////////////////////////////////////////////////////////////////// VREF
// Constants used in setup_vref() are:
//
#define VREF_LOW  0xa0
#define VREF_HIGH 0x80
// Or (with |) the above with a number 0-15
#define VREF_A2   0x40

////////////////////////////////////////////////////////////////// INT
// Interrupt Functions: ENABLE_INTERRUPTS(), DISABLE_INTERRUPTS(),
//                      EXT_INT_EDGE()
//
// Constants used in EXT_INT_EDGE() are:
#define L_TO_H              0x40
#define H_TO_L                 0
// Constants used in ENABLE/DISABLE_INTERRUPTS() are:
#define GLOBAL                    0x0BC0
#define INT_RTCC                  0x0B20
#define INT_RB                    0x0B08
#define INT_EXT                   0x0B10
#define INT_TBE                   0x8C10
#define INT_RDA                   0x8C20
#define INT_TIMER1                0x8C01
#define INT_TIMER2                0x8C02
#define INT_CCP1                  0x8C04
#define INT_LCD                   0x8C80
#define INT_COMP                  0x8C40
#define INT_EEPROM                0x0B40
#define INT_TIMER0                0x0B20

#define uint8 unsigned char
#define uchar unsigned char
//#define uint16 unsigned int 	//      CCS C Compiler
#define uint24 unsigned short long
#define uint32 unsigned long

//----- Register Files------------------------------------------------------
#BYTE INDF=0
#BYTE TMR0=0x01
#BYTE PCL=0x02
#BYTE STATUS=0x03
#BYTE FSR=0x04
#BYTE PORTA=0x05
#BYTE PORTB=0x06
#BYTE PCLATH=0x0A
#BYTE INTCON=0x0B
#BYTE PIR1=0x0C
#BYTE TMR1L=0x0E
#BYTE TMR1H=0x0F
#BYTE T1CON=0x10
#BYTE TMR2=0x11
#BYTE T2CON=0x12
#BYTE CCPR1L=0x15
#BYTE CCPR1H=0x16
#BYTE CCP1CON=0x17
#BYTE RCSTA=0x18
#BYTE TXREG=0x19
#BYTE RCREG=0x1A
#BYTE CMCON=0x1F

#BYTE OPTION_REG=0x81
#BYTE TRISA=0x85
#BYTE TRISB=0x86
#BYTE PIE1=0x8C
#BYTE PCON=0x8E
#BYTE PR2=0x92
#BYTE TXSTA=0x98
#BYTE SPBRG=0x99
#BYTE EEDATA=0x9A
#BYTE EEADR=0x9B
#BYTE EECON1=0x9C
#BYTE EECON2=0x9D
#BYTE VRCON=0x9F

//----- STATUS Bits --------------------------------------------------------

#bit IRP=STATUS.7
#bit RP1=STATUS.6
#bit RP0=STATUS.5
#bit NOT_TO=STATUS.4
#bit NOT_PD=STATUS.3
#bit _Z=STATUS.2
#bit _DC=STATUS.1
#bit _C=STATUS.0

//----- INTCON Bits --------------------------------------------------------

#bit GIE=INTCON.7
#bit PEIE=INTCON.6
#bit T0IE=INTCON.5
#bit INTE=INTCON.4
#bit RBIE=INTCON.3
#bit T0IF=INTCON.2
#bit INTF=INTCON.1
#bit RBIF=INTCON.0

//----- PIR1 Bits ----------------------------------------------------------

#bit EEIF=PIR1.7
#bit CMIF=PIR1.6
#bit RCIF=PIR1.5
#bit TXIF=PIR1.4
#bit CCP1IF=PIR1.2
#bit TMR2IF=PIR1.1
#bit TMR1IF=PIR1.0

//----- T1CON Bits ---------------------------------------------------------
#bit T1CKPS1=T1CON.5
#bit T1CKPS0=T1CON.4
#bit T1OSCEN=T1CON.3
#bit NOT_T1SYNC=T1CON.2
#bit TMR1CS=T1CON.1
#bit TMR1ON=T1CON.0

//----- T2CON Bits ---------------------------------------------------------
#bit TOUTPS3=T2CON.6
#bit TOUTPS2=T2CON.5
#bit TOUTPS1=T2CON.4
#bit TOUTPS0=T2CON.3
#bit TMR2ON=T2CON.2
#bit T2CKPS1=T2CON.1
#bit T2CKPS0=T2CON.0

//----- CCP1CON Bits ---------------------------------------------------------
#bit CCP1X=CCP1CON.5
#bit CCP1Y=CCP1CON.4
#bit CCP1M3=CCP1CON.3
#bit CCP1M2=CCP1CON.2
#bit CCP1M1=CCP1CON.1
#bit CCP1M0=CCP1CON.0

//----- RCSTA Bits ---------------------------------------------------------
#bit SPEN=RCSTA.7
#bit RX9=RCSTA.6
#bit SREN=RCSTA.5
#bit CREN=RCSTA.4
#bit ADEN=RCSTA.3
#bit FERR=RCSTA.2
#bit OERR=RCSTA.1
#bit RX9D=RCSTA.0

//----- CMCON Bits ---------------------------------------------------------

//#bit C2OUT=CMCON.7
//#bit C1OUT=CMCON.6
#bit C2INV=CMCON.5
#bit C1INV=CMCON.4
#bit CIS=CMCON.3
#bit CM2=CMCON.2
#bit CM1=CMCON.1
#bit CM0=CMCON.0

//----- OPTION Bits --------------------------------------------------------

#bit NOT_RBPU=OPTION_REG.7
#bit INTEDG=OPTION_REG.6
#bit T0CS=OPTION_REG.5
#bit T0SE=OPTION_REG.4
#bit PSA=OPTION_REG.3
#bit PS2=OPTION_REG.2
#bit PS1=OPTION_REG.1
#bit PS0=OPTION_REG.0

//----- PIE1 Bits ----------------------------------------------------------

#bit EEIE=PIE1.7
#bit CMIE=PIE1.6
#bit RCIE=PIE1.5
#bit TXIE=PIE1.4
#bit CCP1IE=PIE1.2
#bit TMR2IE=PIE1.1
#bit TMR1IE=PIE1.0

//----- PCON Bits ----------------------------------------------------------

#bit OSCF=PCON.3
#bit NOT_POR=PCON.1
#bit NOT_BO=PCON.0
#bit NOT_BOR=PCON.0

//----- TXSTA Bits ----------------------------------------------------------
#bit CSRC=TXSTA.7
#bit TX9=TXSTA.6
#bit TXEN=TXSTA.5
#bit SYNC=TXSTA.4
#bit BRGH=TXSTA.2
#bit TRMT=TXSTA.1
#bit TX9D=TXSTA.0

//----- EECON1 Bits ---------------------------------------------------------
#bit WRERR=EECON1.3
#bit WREN=EECON1.2
#bit WR=EECON1.1
#bit RD=EECON1.0

//----- VRCON Bits ---------------------------------------------------------

#bit VREN=VRCON.7
#bit VROE=VRCON.6
#bit VRR=VRCON.5
#bit VR3=VRCON.3
#bit VR2=VRCON.2
#bit VR1=VRCON.1
#bit VR0=VRCON.0


/*
 * Useful macros for CCS C compiler
 * Written by Svetoslav Pirev, Nov-2000 - Dec-2005
 */
#define _NOP() #asm nop #endasm
#define _CLRWDT() #asm clrwdt #endasm

#define _MOVLW(value) #asm movlw value #endasm
#define _MOVWF(reg) #asm movwf reg #endasm

#define _ANDLW(value) #asm andlw value #endasm
#define _IORLW(value) #asm iorlw value #endasm

#define _RLF_F(data) #asm rlf data,F #endasm
#define _RLF_W(data) #asm rlf data,W #endasm

#define _RRF_F(data) #asm rrf data,F #endasm
#define _RRF_W(data) #asm rrf data,W #endasm

#define _COMF_F(data) #asm comf data,F #endasm
#define _COMF_W(data) #asm comf data,W #endasm

#define _HIGH(reg) #asm movf reg,W #endasm

#define _MOVF_W(reg) #asm movf reg,W #endasm
#define _MOVF_F(reg) #asm movf reg,F #endasm

#define _ADDWF_F(reg) #asm addwf reg,F #endasm
#define _ADDWF_W(reg) #asm addwf reg,W #endasm

#define _ANDWF_F(reg) #asm andwf reg,F #endasm
#define _ANDWF_W(reg) #asm andwf reg,W #endasm

#define _IORWF_F(reg) #asm iorwf reg,F #endasm
#define _IORWF_W(reg) #asm iorwf reg,W #endasm

#define _SWAPF_F(reg) #asm swapf reg,F #endasm
#define _SWAPF_W(reg) #asm swapf reg,W #endasm

#define _GETFSRF(reg) #asm movf INDF,W	 movwf reg	#endasm

#define _SUBWF_W(reg) #asm subwf reg,W #endasm
#define _SUBWF_F(reg) #asm subwf reg,F #endasm

#define _DECF_F(reg) #asm decf reg,F #endasm
#define _DECF_W(reg) #asm decf reg,W #endasm

#define _INCF_F(reg) #asm incf reg,F #endasm
#define _INCF_W(reg) #asm incf reg,W #endasm

#list
