//********************************************************************
// Subrotines for Write/Read to/from internal EEPROM in processors
// PIC16F627 &  PIC16F628
//********************************************************************

//
// RdE2MByte() read one byte from internal memory from address 'addr'
//
char RdE2MByte(char addr/*Wreg*/)
{
	EEADR = addr;
	RD=1;
	_NOP(); _NOP();
	return EEDATA;
}

//
// RdE2MSeq() read data from EEPROM address 'source' to RAM location at address 'dest'
//
void RdE2MSeq(char* dest,char source,char bytes)
{
	do {
	 *dest++ = RdE2MByte(source++);
	} while(--bytes);
}

//
// WrE2MByte() write 'data' to internal memory at address 'e2maddr'
//
void WrE2MByte(char e2maddr,char data/*Wreg*/)
{
	while( GIE )
		GIE = 0;
	_CLRWDT();
	EEADR = e2maddr;
	EEDATA = data;
	WREN = 1;
	_C = GIE;
	GIE = 0;
	EECON2 = 0x55; EECON2 = 0xAA;
	WR = 1; WREN = 0;
	GIE = _C;
	_NOP(); _NOP();
	while( WR );
	EEIF = 0;
	GIE = 1;
}

//
// WrE2MSeq() write data from RAM location 'source' to internal EEPROM at address 'dest'
//
void WrE2MSeq(char dest,char *source,char bytes)
{
	do {
		WrE2MByte(dest++,*source++);
	} while(--bytes);
}

