#RESERVE 0x7C,0x7D,0x7E,0x7F
#RESERVE 0xFC,0xFD,0xFE,0xFF
#BYTE Wtemp=0x7C
#BYTE STATUStemp=0x7D
#BYTE PCLATHtemp=0x7E
#BYTE FSRtemp=0x7F

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
// subwf -   Wreg  ;       
// sublw -     Wreg,   
//            '1' ( STATUS,C )
//        0,      '1' ( STATUS,C )
//      0,       '1' ( STATUS,Z )
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

//************************************************************
// Push service registers into 'Stack'
//************************************************************
#INLINE
void PushOntoStack(void)
{
#asm
	clrwdt
	movwf 	Wtemp 		// Store W-reg
	swapf 	STATUS,W 	// Swap status to be saved into W
	clrf 	STATUS 		// bank 0, Clears IRP,RP1,RP0
	movwf 	STATUStemp 	// Save status to bank zero STATUS_TEMP
	movf 	PCLATH,W	// pages 1, 2 and/or 3
	movwf 	PCLATHtemp	// Save PCLATH into W
	clrf 	PCLATH 		// Page zero, regardless of current page
	movf 	FSR,W 		// Copy FSR to W
	movwf 	FSRtemp 	// Copy FSR from W to FSR_TEMP
#endasm
}

//************************************************************
// Pop service registers from 'Stack'
//************************************************************
#INLINE
void PopFromStack(void)
{
#asm
	clrf	STATUS			// bank0
	movf	FSRtemp,W		// Restore FSR register
	movwf	FSR
	movf 	PCLATHtemp,W 	// Restore PCLATH
	movwf 	PCLATH 			// Move W into PCLATH
	swapf 	STATUStemp,W 	// Swap STATUS_TEMP register into W
							// (sets bank to original state)
	movwf 	STATUS 			// Move W into STATUS register
	swapf 	Wtemp,F 		// Swap W_TEMP
	swapf 	Wtemp,W 		// Swap W_TEMP into W
//	retfie					// end of interrupt vector
#endasm
}

void Tmr0Handler(void);		// TMR0 overflow vector

#INT_GLOBAL
InterruptHandler()		// Global interrupt vector for PIC16F87X
{
	GIE = 0;
	PushOntoStack();	// Store registers in 'Stack'
	
	if( T0IF )
		Tmr0Handler();
	
	PopFromStack();		// Restore registers from 'Stack'
}

void Tmr0Handler(void)		// TMR0 overflow every 1ms
{
	T0IF = 0;
	_tmr0_overflow = 1;
}
