//
//  lcmath.c
//	         LCMETER
//	version 1.3
//	Written By Svetoslav Pirev, pirev@abv.bg, Aug 2004
//  CCSC C   32        
//	MicroChip.
//  1 nH to 100 mH (most units measure to 150 mH)
//  0.01 pF to 1 uF (most units measure to 1.5 uF)
//

/*
	32 bit float number presentation MicroChip format:
	--------------------------------------------------
		byte+0 	byte+1 	byte+2 	byte+3
		EXP		MSB				LSB
	EXP - 8 bit exponent( real exponent is EXP - 127 )
	MSB - bit 7 - sign bit
	...
	LSB - 23 bit mantisa

	32 bit float number presentation IEEE-754 format:
	-------------------------------------------------
		byte+0 - bit 7 + 7 bit EXP
		byte+1 - bit 7 is bit 0 of EXP, + bits 22,21,20,19,18,17 and 16 of mantissa
		byte+2 - bits 15,14,13,12,11,10,9 and 8 of mantissa
		byte+3 - bits 7,6,5,4,3,2,1 and 0 of mantissa
 */
 
/*
		 
	1)             
	 L1  C8.     freq1. :
		freq1 = 1 / ( 2 * M_PI * SQRT( L1 * C8 ) );

	2)   ,       L1 
	C8 + Ccal.      freq2. :
		freq2 = 1 / ( 2 * M_PI * SQRT( L1 * ( C8 + Ccal ) ) );

	3)          
	.      freq3.

		3a)    :
			freq3 = 1 / ( 2 * M_PI * SQRT( L1 * ( C8 + Cx ) ) );

		   1)   3a),   2)  
		3a),  	   2   Cx,  
		:
					pow ( freq1 / freq3, 2 ) - 1 )
			Cx  =	------------------------------- * Ccal
					pow ( freq1 / freq2, 2 ) - 1 )


		3b)	   :
			freq3 = 1 / ( 2 * M_PI * SQRT( (L1 + Lx) * C8 ) ) );				
		   1)   3b),   1)  
		2),	     2   Lx,  
		:
		
					( pow( F1 / F3, 2 ) - 1 ) * ( pow( F1 / F2, 2 ) - 1 )
			Lx = 	-----------------------------------------------------
							Ccal * pow( 2 * M_PI * F1, 2 )

	    Cx & Lx,  L1  C8     
	,       .  				
	       Ccal.     
	.
 */ 

const float M_PI = 3.14159265;
const float pow_2_pi = ( 2 * M_PI ) * ( 2 * M_PI );
//    10  -   (    
// 100ms,   1s ).     3-     10,
//  Ccal  1-07,   1-09,       
// .
const char DigitPoint  = ',';	// 0x2C
//const char DigitPoint  = '.';	// 0x2E

#define fltTmp fltFreq1Freq2

void MemCpy(char *dst,char *src)
{
	temp = 4;
	do {
		*dst++ = *src++;
	} while( --temp );
}

void Copy_fltRes_to_fltArg1(void)
{
	MemCpy( (char*)&fltArg1, (char*)&fltRes );
}

void Copy_fltRes_to_fltArg2(void)
{
	MemCpy( (char*)&fltArg2, (char*)&fltRes );
}

void MakeFloat(int16 *p)
{
	fltRes = (float)*p;
}

void Sub(void)
{
	fltRes = fltArg1 - fltArg2;
}

void Add(void)
{
	fltRes = fltArg1 + fltArg2;
}

void Div(void)
{
	fltRes = fltArg1 / fltArg2;
}

void Mul(void)
{
	fltRes = fltArg1 * fltArg2;
}

void Calculation1(void)
{
	Div();	// fltRes = fltArg1 / fltArg2;
	Copy_fltRes_to_fltArg1();	// fltArg1 = fltRes;
	Copy_fltRes_to_fltArg2();	// fltArg2 = fltRes;
	Mul();	// fltRes = fltArg1 * fltArg2;
	Copy_fltRes_to_fltArg1();	// fltArg1 = fltRes;
	fltArg2 = 1;
	Sub();	// fltRes = fltArg1 - fltArg2;
}

void Calculation2(void)
{
	// F1_F2
	MakeFloat( &freq1 );
	Copy_fltRes_to_fltArg1();	// fltArg1 = fltRes;
	MakeFloat( &freq2 );
	Copy_fltRes_to_fltArg2();	// fltArg2 = fltRes;
	Calculation1();
	MemCpy( (char*)&fltFreq1Freq2, (char*)&fltRes );	// fltFreq1Freq2 = fltRes;
	// F1_F3
	MakeFloat( &freq1 );
	Copy_fltRes_to_fltArg1();	// fltArg1 = fltRes;
	MakeFloat( &freq3 );
	Copy_fltRes_to_fltArg2();	// fltArg2 = fltRes;
	Calculation1();
	MemCpy( (char*)&fltFreq1Freq3, (char*)&fltRes );	// fltFreq1Freq2 = fltRes;
}

//
//			pow ( F1 / F3, 2 ) - 1 )
//	Cx  =	------------------------ * Ccalibration
//			pow ( F1 / F2, 2 ) - 1 )
//
void CalcCapacitance(void)
{
	Calculation2();
	MemCpy( (char*)&fltArg1, (char*)&fltFreq1Freq3 );
	MemCpy( (char*)&fltArg2, (char*)&fltFreq1Freq2 );
	Div();	// fltRes = fltArg1 / fltArg2;
	Copy_fltRes_to_fltArg1();	// fltArg1 = fltRes;
	fltArg2 = CcalCapacitance;
	Mul();	// fltRes = fltArg1 * fltArg2;
}

//
//			( pow( F1 / F3, 2 ) - 1 ) * ( pow( F1 / F2, 2 ) - 1 )
//	Lx = 	---------------------------------------------------
//				Ccalibration * pow( 2 * M_PI * F1, 2 )
//
void CalcInductance(void)
{
	Calculation2();
	MemCpy( (char*)&fltArg1, (char*)&fltFreq1Freq2 );
	MemCpy( (char*)&fltArg2, (char*)&fltFreq1Freq3 );
	Mul();	// fltRes = fltArg1 * fltArg2;
	MemCpy( (char*)&fltTmp, (char*)&fltRes );
	MakeFloat( &freq1 );
	Copy_fltRes_to_fltArg1();
	Copy_fltRes_to_fltArg2();
	Mul();	// fltRes = fltArg1 * fltArg2;
	Copy_fltRes_to_fltArg1();
	fltArg2 = pow_2_pi;
	Mul();	// fltRes = fltArg1 * fltArg2;
	Copy_fltRes_to_fltArg1();
	fltArg2 = CcalInductance;
	Mul();	// fltRes = fltArg1 * fltArg2;
	MemCpy( (char*)&fltArg1, (char*)&fltTmp );
	Copy_fltRes_to_fltArg2();
	Div();	// fltRes = fltArg1 / fltArg2;
}

void ClrChrBuf(void)
{
	cbuf = 0;
	memset(ChrBuf,0,sizeof(ChrBuf));
}

/*
 * PutChar()      printf      
 *         printf.    
 *         float   ChrBuf
 */
void	PutChar(char ch)
{
	if( cbuf < sizeof(ChrBuf) )
		ChrBuf[cbuf++]=ch;
}

void 	PrintL(int16 l)		// Print in ChrBuf as ASCII characters int16
{
	ClrChrBuf();
	printf(PutChar,"%lu",l);
}

void	Printf(float flt)	// Print in ChrBuf as ASCII characters float number
{
	ClrChrBuf();
	printf(PutChar,"%f",flt);
//	printf(PutChar,"%e",flt);	// exp format is not supported from CCSC C compiler !?!
}

//#define FTOBCDPRECISION 100	// 2 digit after decimal point
#define FTOBCDPRECISION 1000	// 3 digit after decimal point

//
// float to ASCII - 32-bit floating point number 'flt' ( MicroChip format)
// is converted to bcd number - q in bcd.q, r in bcd.r and then to ASCII format
// in buffer ChrBuf[]. Macros FTOBCDPRECISION set how many digit after digit point
// to be show.
//
void 	ftoa(void)
{
char 	dst,
		chr,
		*ptr;

	dst = 0;

	memset(ChrBuf,'0',sizeof(ChrBuf));

	//        .   
	//  7-    1      
	//   MicroChip 
	// if( fltRes < 0.0 )
	FSR = (char*)(&fltRes+1);
	if( bit_test( INDF, 7 ) )		// negative result ?
	{
		ChrBuf[dst++] = '-';
		fltRes = -fltRes;
	} else
		ChrBuf[dst++] = ' ';

	//          BCD 
	Bin16BCD( ( q = (int16)fltRes ), bcd.q );

	//          BCD 
	Bin16BCD( (int16) ( (fltRes - (float)q ) * FTOBCDPRECISION ), bcd.r );

	//             ASCII 

	//       BCD   ASCII 
	bFnd = 0; ptr = bcd.q;
	for( mcnt = 0; mcnt < 6; mcnt++ )
	{
		chr = *ptr;
		if( ! bit_test(mcnt,0) )
			swap(chr);
		chr &= 0x0F;
		chr += '0';

		if( bFnd )
			ChrBuf[dst++] = chr;
		else
			if( chr != '0' )
			{	//       0    
				bFnd = 1;
				ChrBuf[dst++] = chr;
			}

		if( bit_test(mcnt,0) )
			ptr++;
	}
	//          0,  
	//  0   
	FSR = &ChrBuf[0] + dst;
	if( ! bFnd )
	{
		INDF = '0'; FSR++;
	}
	//     ASCII   
	INDF = DigitPoint; FSR++;

	//       BCD   ASCII 
	INDF = ( bcd.r[1] & 0x0F ) + '0'; FSR++;
	INDF = ( bcd.r[2] >> 4 ) + '0'; FSR++;
	INDF = ( bcd.r[2] & 0x0F ) + '0';
}
