#include <16f628.h>
#pragma device *=8
#id	0x0004
#FUSES MCLR, XT, NOWDT, NOPROTECT, BROWNOUT, PUT, NOLVP
#USE Delay(Clock=4000000)	// 4 MHz 
#USE STANDARD_IO(A)
#USE STANDARD_IO(B)
#CASE
#OPT 9

#define PCB_VER3
#include "pin.h"
#include "lcmath.h"

// Strings for LCD display is stored in data EEPROM memory
#ROM 0x2100 = {' ','O','U','T',' ','O','F',' ',' ','R','A','N','G','E',' ',' ',0 }
#ROM 0x2111 = {'n','F',']',0 }	// " nF"
#ROM 0x2115 = {'p','F',']',0 }	// " pF"
#ROM 0x2119 = {'m','H',']',0 }	// " mH"
#ROM 0x211D = {'u','H',']',0 }	// " uH"
#ROM 0x2121 = {'n','H',']',0 }	// " nH"
#ROM 0x2125 = {' ','C',' ','[',0 }	// " C = "
#ROM 0x212A = {' ','L',' ','[',0 }	// " L = "
#ROM 0x212F = {'L','C',' ','M','E','T','E','R','v','e','r',' ',' ','1','.','0',0 }
#ROM 0x2140 = {' ',' ','F','R','E','Q',' ',' ',0}
#ROM 0x2149 = {'u','F',']',0 } 	// " uF"
#ROM 0x214D = {'p','i','r','e','v','@','a','b','v','.','b','g', 0 }
#ROM 0x215A = {  0,  0,  0,  0 }
#ROM 0x2160 = {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0 }
#ROM 0x2170 = {  0,  0,  0,  0,  0,  0,  0,  0 }
#ROM 0x2178 = { 0x61,0x09, 0x70, 0x5F }	// CcalCapacitance = 1E-09
#ROM 0x217C = { 0x67,0x56,0xBF,0x95 }	// CcalInductance = 1E-07

// EEPROM offsets:
#define szOverRange	0
#define sznF 0x11
#define szpF 0x15
#define szmH 0x19
#define szuH 0x1D
#define sznH 0x21
#define szC  0x25
#define szL  0x2A
#define szLCMeter 0x2F
#define szFreq 0x40
#define szuF 0x49
#define szCcalCapacitance 0x78
#define szCcalInductance 0x7C

int16	freq1, 	//        
		freq2, 	//         
		freq3;	//       

float 	fltArg1,	//  1  
		fltArg2,	//  2  
		fltRes,		//     
		fltFreq1Freq2,
	  	fltFreq1Freq3;

float 	CcalCapacitance,
		CcalInductance;

fltbcd	bcd;

int16	q;

char	i,
		j,
		flags,
		temp,
		mcnt,
		cbuf,			//       ChrBuf
		ChrBuf[16];		//       printf()  ftoa()

char	high,
		low;

char	lcd_pos;


#bit _tmr0_overflow=flags.0
#bit bFnd=flags.1
#bit bFtoa=flags.2
#bit bClearDisplay=flags.3
#bit bDigitPoint=flags.4

#include "bcd.c"		// bcd utility
#include "lcmath.c"		// math for C and L calculation
#include "mylcd.c"		// LCD control
#include "eeprom.c"		// DATA memory functions
#include "int.c"		// interrupt vectors

void SendText( char eeprom_offset )
{
	i = 0;
	do {
		temp = RdE2MByte( eeprom_offset++ );
		if( temp == 0 )
			break;
		SendChar( temp );
	} while ( 1 );
}

/*
	Delay100ms()   100ms 
	    4MHz
 */
void Delay100ms(void)
{
	high = 199;
	low = 166;
	do {
		do {} while( --low );
		low = 166;
	} while( --high );
	low = 31;
	do {} while( --low );
	_NOP();
}

void Measure(void)
{
	GIE = 0;
	_tmr0_overflow = 0;
	TMR0 = 0;
	_tmr0_gate = 0;
	high = 0; low = 0;
	GIE = 1;
#ifdef PCB_VER3
	TRISA = 0x18;
#else
	TRISA = 0x11;
#endif
	Delay100ms();
	TRISA = 0x10;
	high = TMR0;
	while( 1 )
	{
		T0SE = 1;
		_NOP();
		T0SE = 0;
		--low;
		if( TMR0 != high )
			break;
	}
	freq3 = high; freq3 <<= 8;
	freq3 |= low;
}

void InitTMR0(void)
{
	// 1:256, T0CKI pin
	OPTION_REG = 0x37;
	TMR0 = 0;
	T0IF = 0;
	T0IE = 1;
}

void ClearRam(void)
{
	FSR = 0x20;
	do {
		INDF = 0; IRP = 1;
		INDF = 0; IRP = 0;
		FSR++;
	} while( ! bit_test(FSR,7) );

	// Clear bank2
	IRP = 1;
	FSR = 0x20;
	do {
		INDF = 0;
	} while( ++FSR < 0x50 );
	IRP = 0;
}

void InitHardware(void)
{
	INTCON = 0;				// Disable all interrupts
	T1CON = 0;				// TMR1 module off
	T2CON = 0;				// TMR2 module off
	CCP1CON = 0;			// CCP1 module off
	CMCON = 0x07;
#ifdef PCB_VER3
	PORTA = 0x04;  			// Relay OFF
#else
	PORTA = 0x02;	 		// Relay OFF
#endif
	PORTB = 0;
	PIR1 = 0;				// Clear interrupts flags
	TRISA = 0x10; TRISB = 0xF0;
	PIE1 = 0;          		// Disable interrupts
	_CLRWDT();
	ClearRam();
	//    CcalCapacitance & CcalInductance  
	//   EEPROM.
	RdE2MSeq( &CcalCapacitance, szCcalCapacitance, 4);
	RdE2MSeq( &CcalInductance, szCcalInductance, 4);
	InitTMR0();
	InitLCD();
	INTCON |= 0xC0;
}


void main(void)
{

	InitHardware();

	while( ! _FREQ )	// LK3
	{	// Display frequency / 10
		Measure();
		if( _tmr0_overflow )
		{
			SetLine1();
			SendText( szOverRange );
		} else
		{
			Bin16BCD( freq3, ChrBuf );
			ClearDisplay(); SendText( szFreq );
			SetLine2();
			SendChar( '0' ); SendChar( '0' );
			SendBCD( ChrBuf[0] );
			SendBCD( ChrBuf[1] );
			SendBCD( ChrBuf[2] );
		}
	}

/*
	ZeroMeter:    L     ,    
	C     .       , 
	       +  -    .
	       .
 */
	ClearDisplay(); SendText( szLCMeter ); // SendText( szPleaseWait );
	Measure();
	Delay100ms(); Delay100ms(); Delay100ms();
	Measure();
	freq1 = freq3;
	_Relay = 0;	// RELAY ON
	Delay100ms(); Delay100ms(); Delay100ms();
	Measure();
	freq2 = freq3;
	_Relay = 1;	// RELAY OFF
	Delay100ms(); Delay100ms(); Delay100ms();
	ClearDisplay();

	//
	// main program loop ...
	//
	while( 1 )
	{
		Measure();
		if( high == 0 )
		{
			SendText( szOverRange );
			bClearDisplay = 1;
			continue;
		}

		if( bClearDisplay )
		{
			bClearDisplay = 0;
			ClearDisplay();
		}
		else
			SetLine1();

		if( _C_OR_L )	// calculate capacitance
		{
			SendText( szC );
			CalcCapacitance();
			if( fltRes < 1E-06 )
			{
				if( fltRes < 1E-09 )
				{
					SendText( szpF );
					fltArg1 = 1E+12;
					j = 2;
				} else
				{
					SendText( sznF );
					fltArg1 = 1E+09;
					j = 3;
				}
			} else
			{
				SendText( szuF );
				fltArg1 = 1E+06;
				j = 2;
			}
			Copy_fltRes_to_fltArg2();
		}else			// calculate inductance
		{
			SendText( szL );
			CalcInductance();
			Copy_fltRes_to_fltArg1();
			if( fltRes < 0.001 )
			{
				SendText( szuH );
				fltArg2 = 1000000.0;
				j = 2;
			} else
			{
				SendText( szmH );
				fltArg2 = 1000.0;
				j = 3;
			}
		}
		Mul();
		ftoa();

		// Show result onto display
		SetLine2();
		i = 0; bDigitPoint = 0; FSR = &ChrBuf[0];
		do {
			if( ! bDigitPoint )
				SendChar( INDF );
			else
			{
				if( j == 0 )
					SendChar( ' ' );
				else {
					--j;
					SendChar( INDF );
				}
			}
			if( INDF == DigitPoint )
			{
				bDigitPoint = 1;
//				j = 2;	// how many digit after digit point to show
			}
			FSR++;
		} while( ++i < 8 );

	}

}
