/*
	_RW      GND.      BusyCheck ,  
	  ,    ,     
	 LCD      .
 */

#define CLR_DISP	0x01			// Clear the Display
#define RET_HOME	0x02			// Set DDRAM to 0 into the adress counter, return cursor ot its original size
#define DISP_ON		0x0C			// Display on
#define DISP_ON_C	0x0E			// Display on, Cursor on
#define DISP_ON_B	0x0F			// Display on, Cursor on, Blink cursor
#define DISP_OFF	0x08			// Display off
#define ENTRY_INC	0x06			// Set entry mode increment, no shift
#define ENTRY_INC_S	0x07			// Set entry mode increment, shift
#define ENTRY_DEC	0x04			// Set entry mode decrement, no shift
#define ENTRY_DEC_S	0x05			// Set entry mode decrement, shift
#define DD_RAM_ADDR	0x80			// Least Significant 7-bit are for address
#define DD_RAM_UL	0x80			// Upper Left coner of the Display

char const cyrletter[64] = 
{
	0x00,0x04,0x0E,0x1F,0x0E,0x04,0x00,0x00,// '' = 0	
	0x1F,0x10,0x10,0x10,0x10,0x10,0x10,0x00,// '' = 1
	0x06,0x0A,0x0A,0x0A,0x0A,0x1F,0x11,0x00,// '' = 2
	0x1E,0x10,0x10,0x1E,0x11,0x11,0x1E,0x00,// '' = 3
	0x11,0x11,0x13,0x15,0x19,0x11,0x11,0x00,// '' = 4
	0x07,0x09,0x09,0x09,0x09,0x09,0x11,0x00,// '' = 5
	0x1F,0x11,0x11,0x11,0x11,0x11,0x11,0x00,// '' = 6 
	0x11,0x11,0x11,0x0F,0x01,0x01,0x01,0x00,// '' = 7
};

char const HexTable[16] = {
	'0','1','2','3',
	'4','5','6','7',
	'8','9','A','B',
	'C','D','E','F'
};

void delaylcd(void)
{
	delay_us(1);
}

void InBus(void)
{
	TRISB |= 0x0F; 
} 

void OutBus(void)
{
	TRISB &= ~0x0F; 
}

void BusyCheck(void)
{
/*
	InBus();
	do {
		_RS = 0; //_RW = 1; 	
		_E = 1; delaylcd();	// Get high nibble of byte
		temp = PORTB;	
		_E = 0; delaylcd(); 
		_E = 1; delaylcd();	// Get low nibble of byte
		DUMMY_READ(PORTB);	
		_E = 0; delaylcd();
	} while( bit_test(temp,7) );
	OutBus();
*/
	delay_us(50);
}


void SetLine1(void);
void SetLine2(void);

void SendChar(char data)
{ 
	BusyCheck();
	// Send upper nibble
	PORTB = data >> 4; delaylcd();
	//_RW = 0; delaylcd(); 
	_RS = 1; delaylcd(); 
	_E = 1; delaylcd(); _E = 0; delaylcd();
	// Send lower nibble
	PORTB = 0x0F & data; delaylcd();
	_E = 1; delaylcd(); _E = 0; delaylcd();
	++lcd_pos;
	if( lcd_pos == 8 )
		SetLine2();
	if( lcd_pos == 16 )
		SetLine1();
}

void SendCmd(char data)
{
	BusyCheck();
	// Send upper nibble
	PORTB = data >> 4; delaylcd();
	//_RW = 0; delaylcd(); 
	_RS = 0; delaylcd();
	_E = 1; delaylcd(); _E = 0; delaylcd();
	// Send lower nibble
	PORTB = 0x0F & data; delaylcd();
	_E = 1; delaylcd(); _E = 0; delaylcd();
}

void SetLine1(void)
{
	SendCmd(DD_RAM_UL);	// display ram upper left corner
	lcd_pos = 0;
}

void SetLine2(void)
{
	SendCmd(DD_RAM_UL|0x40);
	lcd_pos = 8;
}

void ClearDisplay(void)
{
	SendCmd(CLR_DISP);
	delay_ms(2);
	SendCmd(DD_RAM_UL);
	lcd_pos = 0;
}

void HomeDisplay(void)
{
	SendCmd(RET_HOME);
	lcd_pos = 0;
}

//
// Cyr() - cyrilize LCD with Bulgarian letter .
//
void CyrLCD(void)
{	
	SendCmd(0x40);		// SET CG-RAM ADDRESS
	i = 0;
	do {
		SendChar( cyrletter[i] );
	} while ( ++i < 64 );
}

void InitLCD(void)
{ 
	OutBus();
	_E = 0; _RS = 0; //_RW = 0;
	PORTB = 0x03; delaylcd(); _E = 1; delaylcd(); _E = 0; delaylcd();
	delay_ms(5);
	PORTB = 0x03; delaylcd(); _E = 1; delaylcd(); _E = 0; delaylcd();
	delay_us(100);
	PORTB = 0x03; delaylcd(); _E = 1; delaylcd(); _E = 0; delaylcd();
	delay_ms( 2 );
	// Set 4-bit interface
	PORTB = 0x02; delaylcd(); _E = 1; delaylcd(); _E = 0; delaylcd();
	SendCmd(0x28);		// 4-bit interface, 2 lines of 5x7 characters
	SendCmd(DISP_ON);		// Display on, cursor off
//  	SendCmd(DISP_ON_C);		// Display on, cursor on
	SendCmd(CLR_DISP);		// Clear display
	SendCmd(ENTRY_INC);	     	// Set entry mode inc, no shift
	SendCmd(0x80);	     	// Address DDRam upper left
}

void SendBCD(char data)
{ 
	SendChar( (0x0F&swap(data)) + 0x30 );	// Send MSB nibble
	SendChar( (0x0F&swap(data)) + 0x30 ); 	// Send LSB nibble
}

void SendHexPrefix(void)
{
	SendChar(' '); SendChar('0'); SendChar('x');
}

void SendHEX(char data)
{
	SendChar( HexTable[0x0F&swap(data)] );	// Send MSB nibble 
	SendChar( HexTable[0x0F&swap(data)] );	// Send LSB nibble
}

void FillLine1(char fillchar)
{
char i;
	SetLine1();
	i=8; do SendChar(fillchar); while( --i != 0 );
}

void FillLine2(char fillchar)
{
char i;
	SetLine2();
	i=8; do SendChar(fillchar); while( --i != 0 );	
}

